import 'dart:convert';

import 'package:flutter/rendering.dart';
import 'package:flutter_gb_stack_base/flutter_gb_stack_base.dart';

import '../../domain/domain.dart';

class AuthenticationDataModel implements AuthenticationData {
  @override
  final String id;
  @override
  final String token;
  @override
  final User? user;
  @override
  final String? refreshToken;

  static Map<String, dynamic> _decodeToken(String token) {
    List<String> splited = token.split(".");

    String claims = splited[1];
    String decoded = utf8.decode(base64.decode(base64.normalize(claims)), allowMalformed: true);
    Map<String, dynamic> parsed = jsonDecode(decoded);
    return parsed;
  }

  @override
  bool get expired {
    return hasTokenExpired(token);
  }

  static bool hasTokenExpired(String token, [DateTime? compareAgainst]) {
    try {
      Map<String, dynamic> parsed = _decodeToken(token);
      num exp = parsed["exp"] * 1000;
      DateTime expiresIn = DateTime.fromMillisecondsSinceEpoch(
        exp.toInt(),
      );
      return expiresIn.isBefore(compareAgainst ?? DateTime.now());
    } catch (e, s) {
      debugPrint("$e\n$s");
      return true;
    }
  }

  AuthenticationDataModel({
    required this.id,
    required this.token,
    this.user,
    this.refreshToken,
  });

  AuthenticationDataModel copyWith({
    String? id,
    String? token,
    User? user,
    String? refreshToken,
  }) {
    return AuthenticationDataModel(
      id: id ?? this.id,
      token: token ?? this.token,
      user: user ?? this.user,
      refreshToken: refreshToken ?? this.refreshToken,
    );
  }

  factory AuthenticationDataModel.fromToken(String token) {
    Map<String, dynamic> decodedToken = _decodeToken(token);
    return AuthenticationDataModel(
      id: decodedToken["id"] ?? decodedToken["_id"],
      token: token,
    );
  }

  @override
  String toString() {
    return "AuthenticationDataModel(\nid:$id,\ntoken:$token,\nrefreshToken:$refreshToken,\nexpired:$expired,\nuser:$user\n)";
  }
}

// helper function to create a fake instance of the class
AuthenticationDataModel authenticationDataModel() {
  return AuthenticationDataModel(
    id: "1",
    token:
        "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpYXQiOjE2MjMwODQ2MjQsImV4cCI6MTYyMzEwMjYyNH0.HhYgkhQ1SjoWoz9xgP9Yb4rrPCg-X5pwlg20YZt4YtI222",
    refreshToken: "",
    user: null,
  );
}
