import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';

import '../../application/application.dart';

final _getIt = GetIt.instance;

class AuthenticationBasicProvider extends StatefulWidget {
  final Widget child;

  /// Provide a custom [AuthenticationBasicBloc]
  /// useful when using [AuthGuard] since it requires access to the bloc
  final AuthenticationBasicBloc? authBloc;

  /// Automatically dispatch the initial GetSession event for checking if user is Authenticated
  /// defaults to [true]
  final bool autoGetSession;

  const AuthenticationBasicProvider({
    Key? key,
    required this.child,
    this.authBloc,
    this.autoGetSession = true,
  }) : super(key: key);

  @override
  State<AuthenticationBasicProvider> createState() => _AuthenticationBasicProviderState();
}

class _AuthenticationBasicProviderState extends State<AuthenticationBasicProvider> {
  late AuthenticationBasicBloc _bloc;

  @override
  void initState() {
    _bloc = widget.authBloc ??
        AuthenticationBasicBloc(
          authenticationService: _getIt(),
          config: _getIt(),
        );
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) async {
      /// Add a small delay, this will ensure that any normal autoroute page have been initialized
      await Future.delayed(const Duration(milliseconds: 500));
      _bloc.getSession();
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return BlocProvider(
      create: (context) => _bloc,
      lazy: false,
      child: widget.child,
    );
  }
}
