// ignore_for_file: public_member_api_docs, sort_constructors_first
import 'dart:async';

import 'package:dartz/dartz.dart' hide State;
import 'package:flutter/widgets.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_gb_stack_base/flutter_gb_stack_base.dart';

import '../../flutter_gb_authentication_basic.dart';

class AuthenticationBasicBlocListener extends StatefulWidget {
  final Widget child;
  final void Function(BuildContext context, Either<Failure, AuthenticationData> result)? onLoginResult;
  final void Function(BuildContext context, bool authenticated, [AuthenticationData? data])? onAuthChanged;
  final void Function(BuildContext context, String? sessionToken)? onLogout;
  final void Function(BuildContext context, AuthenticationData? sessionToken)? onInitialAuthChecked;
  final void Function(BuildContext context, Either<Failure, AuthenticationData?> result)? onSignupResult;
  final void Function(BuildContext context, Either<Failure, AuthenticationData> result)? onAutoSignupAuthResult;
  final void Function(BuildContext context, Either<Failure, User> result)? onUserFetchResult;
  final void Function(BuildContext context, AuthenticationData authenticationData)? onSessionExpired;
  final void Function(BuildContext context, Either<Failure, User?> result)? onDeleteAccountResult;

  /// Set this to [true] if you want to dispatch callbacks by relying in perviously
  /// dispatched events
  /// This is useful when using this Widget to listen an event that may be already dispatched by the time
  /// it subscribes to the event stream
  final bool useReplayStream;

  const AuthenticationBasicBlocListener({
    Key? key,
    required this.child,
    this.onLoginResult,
    this.onAuthChanged,
    this.onLogout,
    this.onInitialAuthChecked,
    this.onSignupResult,
    this.onAutoSignupAuthResult,
    this.onUserFetchResult,
    this.onSessionExpired,
    this.onDeleteAccountResult,
    this.useReplayStream = false,
  }) : super(key: key);

  @override
  _AuthenticationBasicBlocListenerState createState() => _AuthenticationBasicBlocListenerState();
}

class _AuthenticationBasicBlocListenerState extends State<AuthenticationBasicBlocListener> {
  @override
  Widget build(BuildContext context) {
    return BlocEventListener<AuthenticationBasicBloc, AuthenticationBasicEvent>(
      onEvent: (context, event) {
        // Some events could produce unwanted behavior if dispatched and handled before the bloc does, so let's
        // delay it a little 1 frame
        WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
          if (!mounted) return;
          event.mapOrNull(
            // getSession: getSession,
            // loginWithCredentials: loginWithCredentials,
            loginWithCredentialsRes: (event) {
              widget.onLoginResult?.call(context, event.response);
            },
            authChanged: (event) {
              widget.onAuthChanged?.call(context, event.authenticated, event.data);
            },
            logout: (event) async {
              await Future.delayed(const Duration(milliseconds: 10));
              widget.onLogout?.call(context, event.sessionToken);
            },
            initialAuthChecked: (event) {
              widget.onInitialAuthChecked?.call(context, event.data);
            },
            signupWithCredentialsResult: (event) {
              widget.onSignupResult?.call(context, event.result);
            },
            autoSignupAuthResult: (event) {
              widget.onAutoSignupAuthResult?.call(context, event.result);
            },
            fetchAuthUserDataResult: (event) {
              widget.onUserFetchResult?.call(context, event.result);
            },
            sessionExpired: (event) {
              widget.onSessionExpired?.call(context, event.authData);
            },
            deleteAccountResult: (event) {
              widget.onDeleteAccountResult?.call(context, event.result);
            },
            // orElse: () {},
          );
        });
      },
      child: widget.child,
    );
  }
}
