import 'package:auto_route/auto_route.dart';
import 'package:flutter_gb_authentication_basic/flutter_gb_authentication_basic.dart';
import 'package:rxdart/rxdart.dart';

class AuthGuard extends AutoRouteGuard {
  final void Function(StackRouter router)? onUnauthorized;
  final AuthenticationBasicBloc authBloc;

  AuthGuard({
    required this.onUnauthorized,
    required this.authBloc,
  });

  @override
  void onNavigation(NavigationResolver resolver, StackRouter router) async {
    // final authBloc = await GetIt.instance.get<AuthenticationBasicBloc>();

    // the navigation is paused until resolver.next() is called with either
    // true to resume/continue navigation or false to abort navigation
    if (authBloc.state.authenticated && !authBloc.state.authenticationData!.expired) {
      // if user is authenticated we continue
      return resolver.next(true);

      // lets check if the session has expired
    } else if (authBloc.state.authenticated && authBloc.state.authenticationData!.expired) {
      // if the user configured the refreshApiEndpoint, lets wait for it to return a refreshed session or failure
      if (authBloc.config.refreshTokenAPIendpoint != null) {
        await authBloc.eventsReplay
            .where(
              (event) {
                return event is RefreshTokenSuccess || event is RefreshTokenFailure;
              },
            )
            .map((event) => null)
            .timeout(const Duration(seconds: 10))
            // lets give it a moment so the bloc/service set the new session on state
            .delay(const Duration(milliseconds: 100))
            // do nothing on error we still be checking if session have changed
            .onErrorReturn(null)
            .first;
        if ((authBloc.state.authenticationData?.expired ?? true)) {
          resolver.next(false);
          return onUnauthorized?.call(router);
        } else {
          return resolver.next(true);
        }
      } else {
        resolver.next(false);
        return onUnauthorized?.call(router);
      }
    } else {
      resolver.next(false);
      // we redirect the user to our login page
      return onUnauthorized?.call(router);
    }
  }
}
